"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.logger = exports.logError = exports.loggerMiddleware = void 0;
const process_1 = require("process");
const winston_1 = __importDefault(require("winston"));
const express_winston_1 = __importDefault(require("express-winston"));
const ecs_winston_format_1 = __importDefault(require("@elastic/ecs-winston-format"));
let configs;
try {
    configs = require(process_1.cwd() + '/logger.config');
}
catch {
    throw Error("Logger configs could not be found on app root's directory.");
}
function filterRequestHeaders(confidentialHeaders = []) {
    return (req, propName) => {
        if (propName !== 'headers')
            return req[propName];
        const filteredHeaders = Object.keys(req.headers)
            .filter(key => !confidentialHeaders.includes(key))
            .reduce((acc, key) => {
            acc[key] = req.headers[key];
            return acc;
        }, {});
        return filteredHeaders;
    };
}
const routeFilter = (req, res) => {
    const { url } = req;
    const routesToBeFiltered = configs.ignoredRoutes || [];
    const shouldRouteBeFiltered = shouldFilterRoute(url, routesToBeFiltered);
    return shouldRouteBeFiltered;
};
function shouldFilterRoute(url, routesToBeFiltered) {
    const shouldRouteBeFiltered = routesToBeFiltered.some(route => {
        const regExp = generateRouteRegex(route);
        const matches = new String(url).match(regExp);
        return !!(matches === null || matches === void 0 ? void 0 : matches.length);
    });
    return shouldRouteBeFiltered;
}
function getTransports() {
    const transports = [];
    if (configs.console) {
        transports.push(new winston_1.default.transports.Console(configs.console));
    }
    if (configs.file) {
        transports.push(new winston_1.default.transports.File(configs.file));
    }
    return transports;
}
function isRequestInfo(info) {
    const metadata = info.meta;
    return (metadata === null || metadata === void 0 ? void 0 : metadata.req) && (metadata === null || metadata === void 0 ? void 0 : metadata.res) ? true : false;
}
function generateRouteRegex(route) {
    const splitedRoute = route.split('/').filter(i => i !== '');
    const regexString = splitedRoute
        .map(i => {
        return (i === null || i === void 0 ? void 0 : i.charAt(0)) === ':' ? '(.)+' : i;
    })
        .join('/');
    const regExp = new RegExp('^/' + regexString + '$', 'g');
    return regExp;
}
function shouldRouteBeFiltered(request, blacklistedRoutes) {
    return blacklistedRoutes.some(({ route, method }) => {
        const regExp = generateRouteRegex(route);
        const matches = new String(request.url).match(regExp);
        return ((matches === null || matches === void 0 ? void 0 : matches.length) && request.method.toLowerCase() === method.toLowerCase());
    });
}
function filterBlacklistedBodysByRoute(info) {
    var _a, _b;
    const blacklistedResponseRoutes = configs.responseBlacklistBodyRoutes || [];
    const blacklistedRequestRoutes = configs.requestBlacklistBodyRoutes || [];
    const request = info.meta.req;
    const shouldResponseBeFiltered = shouldRouteBeFiltered(request, blacklistedResponseRoutes);
    const shouldRequestBeFiltered = shouldRouteBeFiltered(request, blacklistedRequestRoutes);
    if (shouldResponseBeFiltered) {
        (_a = info.meta.res) === null || _a === void 0 ? true : delete _a.body;
    }
    if (shouldRequestBeFiltered) {
        (_b = info.meta.req) === null || _b === void 0 ? true : delete _b.body;
    }
    return info;
}
function formatBlacklistedBody() {
    return winston_1.default.format(info => {
        const isRequest = isRequestInfo(info);
        if (isRequest) {
            const filteredInfo = filterBlacklistedBodysByRoute(info);
            return filteredInfo;
        }
        return info;
    })();
}
function createLoggerMiddleware() {
    express_winston_1.default.requestWhitelist.push('body', ...(configs.requestWhitelist || []));
    express_winston_1.default.responseWhitelist.push('body', ...(configs.responseWhitelist || []));
    return express_winston_1.default.logger({
        statusLevels: true,
        transports: getTransports(),
        requestFilter: filterRequestHeaders(configs.confidentialHeaders || []),
        ignoreRoute: routeFilter,
        format: winston_1.default.format.combine(formatBlacklistedBody(), ecs_winston_format_1.default({ convertReqRes: true })),
    });
}
function createErrorLoggerMiddleware() {
    return express_winston_1.default.errorLogger({
        format: ecs_winston_format_1.default({ convertReqRes: true }),
        transports: getTransports(),
        requestFilter: filterRequestHeaders(configs.confidentialHeaders || []),
    });
}
exports.loggerMiddleware = createLoggerMiddleware();
exports.logError = createErrorLoggerMiddleware();
exports.logger = winston_1.default.createLogger({
    format: ecs_winston_format_1.default({ convertReqRes: true }),
    transports: getTransports(),
    exitOnError: false,
});
exports.logger.stream = Object.assign(exports.logger.stream, {
    write(message) {
        exports.logger.info(JSON.parse(message));
    },
});
